/* ############################################################# *

csv2bin: convert a stars.csv file to a binary file
version 1.0

Written by: Vurt, <sbenoit@mnsi.net>
Last modified: 2000/01/15

usage 1: csv2bin <csv file> <bin file>
usage 2: csv2bin <bin file>
usage 3: csv2bin



Disclaimer:  Neither Particle Systems nor Infogrames are
responsible for this code, or anything it does to your
machine, and cannot be held accountable.  For that matter,
I'm not accountable for anything this code does either, etc.

Feel free to modify this code, and distribute it, but please
credit the original author.  If you make any really good
mods, or find and fix a bug, please send me a copy of what
you did so that I can incorporate it into the original
version.

Finally, you are prohibited from making any money from my work.


Special thanks go out to Particle Systems, for obvious reasons,
and to Nanoprobe for getting me started on this, and for his
help debugging.

 * ############################################################# */


#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAX_FILENAME_LENGTH 127
#define MAX_LINELENGTH 256
#define NO_FIELDS_PER_LINE 8
#define STAR_NAME_SIZE 20
#define DEFAULT_INFILENAME  "stars.csv"
#define DEFAULT_OUTFILENAME "stars.bin"

typedef struct StarData StarData;
struct StarData {
  float x;
  float y;
  float z;
  char name[STAR_NAME_SIZE+1];
  float apparent_mag;
  float absolute_mag;
  unsigned char system;
  unsigned long flags;
};

main(int argc, char *argv[]);
void clearstar(StarData *star);
int readstar(StarData *star, FILE *fp);
int writestar(StarData *star, FILE *fp);
int getline(FILE *fp, char *line);
int parseline(char *line, StarData *star);
void replace(char *p, char q, char r);


// it all starts with main...
main(int argc, char *argv[])
{
  StarData star;
  StarData *pstar = &star;

  FILE *fpi;
  FILE *fpo;
  char inputfile[MAX_FILENAME_LENGTH+1] = DEFAULT_INFILENAME;
  char outputfile[MAX_FILENAME_LENGTH+1] = DEFAULT_OUTFILENAME;
  int eof = 0;

  if (1 == argc) // no input, default files
  {
    ; // nothing to do
  }
  else if (2 == argc) // assume only output file specified
  {
	strncpy(outputfile, argv[1], MAX_FILENAME_LENGTH);
  }
  else if (3 == argc) // input and output files specified
  {
    strncpy(inputfile, argv[1], MAX_FILENAME_LENGTH);
    strncpy(outputfile, argv[2], MAX_FILENAME_LENGTH);
  }
  else // too many arguments
  {
    printf("csv2bin: too many arguments\n");
    exit(1);
  }

  strcat(inputfile, '\0');
  strcat(outputfile, '\0');

  // open the inputfile and make sure it's valid
  fpi = fopen(inputfile, "r");

  if (!fpi)
  {
	printf("csv2bin: unable to open %s\n", inputfile);
	exit(2);
  }

  // open the outputfile and make sure it's valid
  fpo = fopen(outputfile, "w+b");

  if (!fpo)
  {
	printf("csv2bin: unable to open %s\n", outputfile);
	exit(3);
  }

  printf("csv2bin: building %s...  ", outputfile);

  // while not the end of file, convert
  while (!eof)
  {
	clearstar(pstar);
	eof = readstar(pstar, fpi);
	if (!eof) writestar(pstar, fpo);
  }

  printf("finished!\n");

  // clean up
  fclose(fpi);
  fclose(fpo);
}


// initialize the star
void clearstar(StarData *star)
{
  star->x = 0.0;
  star->y = 0.0;
  star->z = 0.0;
  memset(star->name, '\0', STAR_NAME_SIZE+1);
  star->apparent_mag = 0.0;
  star->absolute_mag = 0.0;
  star->system = '\0';
  star->flags = 0;
}


// read the star from the specified file, return 1 if at the EOF
int readstar(StarData *star, FILE *fp)
{
  char line[MAX_LINELENGTH];

  if (0 == getline(fp, line))
  {
    return 1;
  }
  else
  {
    parseline(line, star);
    return 0;
  }
}


// write the star to the specified file
int writestar(StarData *star, FILE *fp)
{
  fwrite(&star->x, sizeof(float), 1, fp);
  fwrite(&star->y, sizeof(float), 1, fp);
  fwrite(&star->z, sizeof(float), 1, fp);
  fwrite(&star->name, STAR_NAME_SIZE, 1, fp);
  fwrite(&star->apparent_mag, sizeof(float), 1, fp);
  fwrite(&star->absolute_mag, sizeof(float), 1, fp);
  fwrite(&star->system, sizeof(char), 1, fp);
  fwrite(&star->flags, sizeof(long), 1, fp);
}


// read in a line, return its length
// modified from "The C Programming Language, 2nd Ed.", Kernighan & Ritchie, p. 165
int getline(FILE *fp, char *line)
{
  if (fgets(line, MAX_LINELENGTH, fp) == NULL)
    return 0;
  else
    return strlen(line);
}


// parse line from CSV into its star data values
// simple token-parsing algorithm from "The Practice of Programming", Kernighan & Pike, p. 88
int parseline(char *line, StarData *star)
{
  int n;
  char *p, *q;
  char *field[NO_FIELDS_PER_LINE];

  n = 0;
  for (q = line; (p = strtok(q, ",\n\r\0")) != NULL; q = NULL)
  	field[n++] = p;

  star->x = atof(field[0]);
  star->y = atof(field[1]);
  star->z = atof(field[2]);

  strncpy(star->name, field[3], STAR_NAME_SIZE);
  strcat(star->name, '\0');
  replace(star->name, '*', ',');

  star->apparent_mag = atof(field[4]);
  star->absolute_mag = atof(field[5]);
  star->system = atoi(field[6]);
  star->flags = atol(field[7]);
}


// replace any instance of q in the string p with r
void replace(char *p, char q, char r)
{
  int i;

  for (i = 0; i < strlen(p); i++)
    if (p[i] == q)
      p[i] = r;

}